#version 430

layout (location = 0) in vec2 mesh_xy;
layout (location = 1) in vec2 mesh_uv;

layout (location = 2) in vec2 instance_xbasis;
layout (location = 3) in vec2 instance_ybasis;
layout (location = 4) in vec2 instance_origin;
layout (location = 5) in vec4 instance_tint;
layout (location = 6) in float instance_depth;
layout (location = 7) in vec4 instance_rect;

layout (binding = 0) uniform Projection {
	mat4 projection_matrix;
};

layout (location = 0) out vec4 color;
layout (location = 1) out vec2 uv;

void main() {
	// Calculate the world position of the vertex
	const vec2 world_position = instance_origin + mesh_xy.x * instance_xbasis + mesh_xy.y * instance_ybasis;
	const vec2 projected_position = (projection_matrix * vec4(world_position, 0, 1)).xy;

	// Set the position of the vertex in clip space
	gl_Position = vec4(projected_position, instance_depth, 1.0);
	color = instance_tint;

	// Calculate the width and height from left, top, right, bottom configuration
	const vec2 rect_pos = instance_rect.xy; // left, top
	const vec2 rect_size = instance_rect.zw - instance_rect.xy; // right - left, bottom - top

	// Calculate the adjusted UV coordinates using the instance_rect
	uv = rect_pos + (mesh_uv * rect_size);
}

